@@ -9,11 +9,6 @@ class AwsAppInstanceIdentifierCreator(IAppInstanceIdentifierCreator):
         super().__init__(logger)
 
     def create(self, app_name: str, ip_address: str, additional_data: {}) -> AppInstanceIdentifier:
-        self._logger.info("entered - app_name: '{}', address: {}, additional_data: {}".format(
-            app_name,
-            ip_address,
-            additional_data))
-
         if 'instance-id' not in additional_data:
             raise Exception('no instance-id found in additional_data send the the request')
         return AppInstanceIdentifier(name=app_name, infra_id=additional_data['instance-id'])
