@@ -1,3 +1,3 @@
 from .logs import LogEntry, ICloudLogger
-from .dummy_logger import DummyLogger
+from .file_logger import FileLogger
 from .cloud_logger_factory import CloudLoggerFactory
