@@ -5,15 +5,17 @@ from logging import Logger
 
 from sidecar.aws_session import AwsSession
 from .. import cloud_logger
-from . import ICloudLogger, DummyLogger
+from . import ICloudLogger, FileLogger
 
 
 class CloudLoggerFactory(object):
-    def __init__(self, config: dict, logger: Logger, logger_session: AwsSession):
+    def __init__(self, config: dict, logger: Logger, logger_session: AwsSession, file_logger: FileLogger):
         self.config = config
         self.logger = logger
         self.cloud_logger = None
         self.logger_session = logger_session
+        self.file_logger = file_logger
+
         provider = config["provider"]
         for path, pkg_name, is_package in pkgutil.iter_modules(cloud_logger.__path__):
             if is_package and pkg_name == provider:
@@ -25,6 +27,6 @@ class CloudLoggerFactory(object):
 
     def create_instance(self):
         if self.cloud_logger is not None:
-            return self.cloud_logger(self.config, self.logger, self.logger_session)
+            return self.cloud_logger(self.config, self.logger, self.logger_session, self.file_logger)
         else:
-            return DummyLogger(self.config)
+            return FileLogger(self.config)
