@@ -5,6 +5,7 @@ from typing import List, Dict
 from blinker import Signal
 
 from sidecar.const import AppNetworkStatus
+from sidecar.utils import CallsLogger
 
 
 class AppHealthCheckState:
@@ -15,11 +16,8 @@ class AppHealthCheckState:
         self._logger = logger
         self._app_states = {app_name: AppNetworkStatus.PENDING for app_name in app_names}  # type: Dict[str, str]
 
+    @CallsLogger.wrap
     def set_app_state(self, app_name: str, status: str):
-        self._logger.info("entered - app_name: '{}', status: '{}'".format(
-            app_name,
-            status))
-
         with self._lock:
             self._app_states[app_name] = status
 
@@ -30,17 +28,14 @@ class AppHealthCheckState:
         if all_apps_complete:
             self.on_apps_deployment_complete.send(self)
 
+    @CallsLogger.wrap
     def all_complete_with_success(self) -> bool:
-        self._logger.info("entered")
-
         with self._lock:
             return all(status == AppNetworkStatus.COMPLETED
                        for status
                        in self._app_states.values())
 
+    @CallsLogger.wrap
     def get_apps_state(self, app_names: List[str]) -> Dict[str, str]:
-        self._logger.info("entered - app_names: '{}'".format(
-            ", ".join(app_names)))
-
         with self._lock:
             return {k: v for k, v in self._app_states.items() if k in app_names}
