@@ -10,5 +10,6 @@ class KubSandboxDeploymentEndUpdater(ISandboxDeploymentEndUpdater):
         self.kub_api_service = kub_api_service
 
     def _set_deployment_end_details(self, deployment_end_status: str):
-        annotations = {Const.SANDBOX_DEPLOYMENT_END_STATUS: deployment_end_status}
+        annotations = {Const.SANDBOX_DEPLOYMENT_END_STATUS: deployment_end_status,
+                       Const.SANDBOX_DEPLOYMENT_END_STATUS_v2: deployment_end_status}
         self.kub_api_service.update_namespace(annotations)
