@@ -22,5 +22,6 @@ class KubSandboxStartTimeUpdater(ISandboxStartTimeUpdater):
         self.date_time_provider = date_time_provider
 
     def _on_health_check_done(self):
-        annotations = {Const.SANDBOX_START_TIME: str(self.date_time_provider.get_current_time_utc())}
+        time = str(self.date_time_provider.get_current_time_utc())
+        annotations = {Const.SANDBOX_START_TIME: time, Const.SANDBOX_START_TIME_v2: time}
         self.kub_api_service.update_namespace(annotations)
