@@ -1,13 +1,14 @@
 import json
 import logging.config
 import os
+
 from datetime import datetime
 from typing import List, Tuple
 
 from blinker import Namespace
-from flask import Flask, request, request_started, request_finished, got_request_exception
-from flask import jsonify
+from flask import Flask, request
 
+from werkzeug.exceptions import HTTPException
 from sidecar.app_configuration_start_policy import AppConfigurationStartPolicy
 from sidecar.app_instance_config_status_event_reporter import AppInstanceConfigStatusEventReporter
 from sidecar.app_instance_event_handler import AppInstanceEventHandler
@@ -24,9 +25,9 @@ from sidecar.aws_sandbox_deployment_end_updater import AwsSandboxDeploymentEndUp
 from sidecar.aws_sandbox_start_time_updater import AwsSandboxStartTimeUpdater
 from sidecar.aws_session import AwsSession
 from sidecar.aws_status_maintainer import AWSStatusMaintainer
-from sidecar.cloud_logger import ICloudLogger, LogEntry, CloudLoggerFactory
+from sidecar.cloud_logger import ICloudLogger, LogEntry, CloudLoggerFactory, FileLogger
 from sidecar.const import Const, DateTimeProvider
-from sidecar.azure.azure_sidecar_api_initializer import AzureSidecarApiInitializer
+from sidecar.azure_clp.azure_sidecar_api_initializer import AzureSidecarApiInitializer
 from sidecar.file_system import FileSystemService
 from sidecar.filters import LoggingFilter
 from sidecar.health_check.app_health_check_monitor import AppHealthCheckMonitor
@@ -43,11 +44,15 @@ from sidecar.kub_sandbox_deployment_end_updater import KubSandboxDeploymentEndUp
 from sidecar.kub_sandbox_start_time_updater import KubSandboxStartTimeUpdater
 from sidecar.kub_token_provider import KubTokenProvider, FakeKubTokenProvider
 from sidecar.messaging_service import MessagingService, MessagingConnectionProperties
+from sidecar.request_logger import RequestLogger
 from sidecar.sandbox_deployment_state_tracker import SandboxDeploymentStateTracker
+from sidecar.sandbox_end_deployment_notifier import SandboxEndDeploymentNotifier
 from sidecar.sandbox_start_time_updater import ISandboxStartTimeUpdater
+from sidecar.utils import CallsLogger
 
 app = Flask(__name__)
 cloud_logger = None  # type: ICloudLogger
+_request_logger = None # type: RequestLogger
 _app_instance_health_check_monitor = None  # type: AppInstanceHealthCheckMonitor
 _app_configuration_start_policy = None  # type: AppConfigurationStartPolicy
 _app_instance_identifier_creator = None  # type: IAppInstanceIdentifierCreator
@@ -60,6 +65,7 @@ _apps_configuration_end_tracker = None  # type: AppsConfigurationEndTracker
 _app_health_check_monitor = None  # type: AppHealthCheckMonitor
 _app_status_maintainer = None  # type: AppStatusMaintainer
 _app_instance_health_check_executor = None  # type: HealthCheckExecutor
+_app_sandbox_end_deployment_notifier = None  # type: SandboxEndDeploymentNotifier
 
 
 def _load_config():
@@ -121,10 +127,6 @@ def get_status_updater(config_json,
     apps_json = config_json['apps']
     app_requests = _convert_apps_json_to_app_requests(apps_json=apps_json)
 
-    messaging_props = _convert_messaging_json_to_connection_props(config_json['messaging'])
-
-    messaging_service = MessagingService(messaging_props, logger)
-
     app_instance_status_event_reporter = AppInstanceConfigStatusEventReporter(
         app_instance_event_handler=app_instance_event_handler, logger=logger)
     time_provider = DateTimeProvider()
@@ -159,7 +161,6 @@ def get_status_updater(config_json,
             apps_configuration_end_tracker=_apps_configuration_end_tracker,
             sandbox_deployment_end_updater=sandbox_deployment_end_updater,
             sandbox_id=sandbox_id,
-            messaging_service=messaging_service,
             space_id=space_id)
 
         _app_status_maintainer = AppStatusMaintainer(logger=logger,
@@ -200,7 +201,6 @@ def get_status_updater(config_json,
             apps_configuration_end_tracker=_apps_configuration_end_tracker,
             sandbox_deployment_end_updater=sandbox_deployment_end_updater,
             sandbox_id=sandbox_id,
-            messaging_service=messaging_service,
             space_id=space_id)
 
         _app_status_maintainer = AppStatusMaintainer(logger=logger,
@@ -229,7 +229,6 @@ def get_status_updater(config_json,
             AzureSidecarApiInitializer(sandbox_id=sandbox_id,
                                        space_id=space_id,
                                        app_requests=app_requests,
-                                       messaging_service=messaging_service,
                                        app_instance_status_event_reporter=app_instance_status_event_reporter,
                                        app_health_check_state=_app_health_check_state,
                                        logger=logger,
@@ -242,104 +241,92 @@ def get_status_updater(config_json,
 @app.route("/")
 def hello():
     logger.info("start")
-
-    return "welcome cs18:sidecar-api"
+    return "welcome cs18:sidecar-api", 200
 
 
 @app.route("/application/<string:app_name>/health-check", methods=['POST'])
 def start_health_check(app_name: str):
-    logger.info("entered with - app_name: {}, request: {}".format(
-        app_name,
-        request.json))
-    try:
-        ip_address = request.remote_addr
 
-        req = request.get_json()
-        script_name = req['script']
-        additional_data = req.get('additional_data')
+    ip_address = request.remote_addr
+    req = request.get_json()
+    script_name = req['script']
+    additional_data = req.get('additional_data')
 
-        logger.info("start_health_check for app '{}', address: {}, request: {}".format(app_name, ip_address, req))
-        app_instance_identifier = _app_instance_identifier_creator.create(app_name=app_name,
-                                                                          ip_address=ip_address,
-                                                                          additional_data=additional_data)
-        logger.info("created identifier for '{APP}' on '{IP}': '{APP_ID}'".format(APP=app_name,
-                                                                                  IP=ip_address,
-                                                                                  APP_ID=app_instance_identifier))
-
-        _app_instance_health_check_monitor.start(address=ip_address,
-                                                 identifier=app_instance_identifier,
-                                                 script_name=script_name)
-
-        # fail here if validation fails ONLY
-        return jsonify(success=True), 202
-    except Exception as exc:
-        logger.exception("Failed to start_health_check for app '{0}'".format(app_name))
-        return jsonify(success=False, error=str(exc)), 500
+    app_instance_identifier = _app_instance_identifier_creator.create(app_name=app_name,
+                                                                      ip_address=ip_address,
+                                                                      additional_data=additional_data)
+    _app_instance_health_check_monitor.start(address=ip_address,
+                                             identifier=app_instance_identifier,
+                                             script_name=script_name)
+    # fail here if validation fails ONLY
+    return '', 202
 
 
 @app.route('/application/<app_name>/<instance_id>/logs/<topic>', methods=['POST'])
 def write_log(app_name: str, instance_id: str, topic: str):
-    try:
-        events = [(datetime.fromtimestamp(item["date"]), item["line"]) for item in request.json]
-        log_entry = LogEntry(app_name, instance_id, topic, events)
-        cloud_logger.write(log_entry)
 
-        return jsonify(success=True), 200
-    except Exception as exc:
-        logger.exception(
-            "writing log error, entered with - app_name: {}, instance_id: {}, topic: {}, request: {}".format(
-                app_name, instance_id, topic,
-                request.json))
-        return jsonify(success=False, error=str(exc)), 500
+    events = [(datetime.fromtimestamp(item["date"]), item["line"]) for item in request.json]
+    log_entry = LogEntry(app_name, instance_id, topic, events, "Events")
+    cloud_logger.write(log_entry)
+
+    return '', 200
 
 
 @app.route('/application/<string:app_name>/<string:instance_id>/event', methods=['POST'])
 def report_app_instance_event(app_name: str, instance_id: str):
-    logger.info(
-        "entered with - app_name: {}, instance_id: {}, request: {}".format(app_name, instance_id, request.json))
-    try:
-        data_json = request.get_json()
-        event = data_json['event']
-        if event not in AppInstanceEvents.ALL:
-            error_message = "'{EVENT}' is not one of the known event types".format(EVENT=event)
-            return jsonify(success=False, error=error_message), 400
 
-        app_instance_identifier = _app_instance_identifier_creator.create_from_instance_id(app_name=app_name,
-                                                                                           instance_id=instance_id)
-        _app_instance_event_handler.report_event(app_instance_identifier=app_instance_identifier,
-                                                 app_instance_event=event)
-        return jsonify(success=True), 200
-    except Exception as exc:
-        logger.exception("Failed to report_app_instance_event for app '{0}'".format(app_name))
-        return str(exc), 500
+    data_json = request.get_json()
+    event = data_json['event']
+    if event not in AppInstanceEvents.ALL:
+        error_message = "'{EVENT}' is not one of the known event types".format(EVENT=event)
+        return error_message, 400
+
+    app_instance_identifier = _app_instance_identifier_creator.create_from_instance_id(app_name=app_name,
+                                                                                       instance_id=instance_id)
+    _app_instance_event_handler.report_event(app_instance_identifier=app_instance_identifier,
+                                             app_instance_event=event)
+    return '', 200
 
 
 @app.route("/application/<string:app_name>/config-start-status", methods=['GET'])
 def get_application_configuration_start_status(app_name: str):
-    logger.info(
-        "entered with - app_name: {}, request: {}".format(app_name, request.json))
-    try:
-        start_status = _app_configuration_start_policy.get_app_configuration_start_status(app_name)
-        return start_status, 200
-    except Exception as exc:
-        logger.exception("Failed to get_application_configuration_start_status for app '{0}'".format(app_name))
-        return str(exc), 500
+
+    start_status = _app_configuration_start_policy.get_app_configuration_start_status(app_name)
+    return start_status, 200
 
 
 @app.route("/is-api-alive", methods=['GET'])
 def get_api_is_alive(app_name: str):
-    logger.info(
-        "entered with - app_name: {}, testing if API is alive".format(app_name))
     return '', 200
 
+@app.route("/application/<string:app_name>/whoami", methods=['POST'])
+def get_who_am_i(app_name: str):
+
+    ip_address = request.remote_addr
+    req = request.get_json()
+    additional_data = req.get('additional_data')
+
+    app_instance_identifier = _app_instance_identifier_creator.create(app_name=app_name,
+                                                                      ip_address=ip_address,
+                                                                      additional_data=additional_data)
+    return app_instance_identifier.infra_id, 200
+
+@app.errorhandler(Exception)
+def unhandled_exception(e):
+    status = e.code if isinstance(e, HTTPException) else 500
+    _request_logger.log_request_failed(e, status)
+    return str(e), status
+
 
 def run():
     global _app_instance_health_check_monitor
     global logger
     global cloud_logger
+    global _request_logger
     global _app_instance_event_handler
     global _app_health_check_monitor
     global _app_instance_health_check_executor
+    global _app_sandbox_end_deployment_notifier
 
     dir_name = os.path.dirname(os.path.abspath(__file__))
 
@@ -352,9 +339,8 @@ def run():
 
         logger.info('starting sidecar from pypi')
 
-        request_started.connect(app_request_started, app)
-        request_finished.connect(app_request_ended, app)
-        got_request_exception.connect(app_request_exception, app)
+        _request_logger = RequestLogger(app, logger, [write_log])
+        CallsLogger.set_logger(logger)
 
         provider = config_json["provider"]
         sandbox_id = config_json['sandbox_id']
@@ -363,7 +349,7 @@ def run():
             region_name = config_json['region_name']
             aws_session = AwsSession(region_name=region_name, stack_name='sandbox-{}'.format(sandbox_id), logger=logger)
 
-        cloud_logger_factory = CloudLoggerFactory(config_json, logger, aws_session)
+        cloud_logger_factory = CloudLoggerFactory(config_json, logger, aws_session, FileLogger(config_json))
         cloud_logger = cloud_logger_factory.create_instance()
 
         _app_instance_event_handler = AppInstanceEventHandler(cloud_logger=cloud_logger,
@@ -405,6 +391,16 @@ def run():
                                                           apps_configuration_end_tracker=_apps_configuration_end_tracker,
                                                           logger=logger)
 
+        messaging_props = _convert_messaging_json_to_connection_props(config_json['messaging'])
+
+        messaging_service = MessagingService(messaging_props, logger)
+
+        _app_sandbox_end_deployment_notifier = SandboxEndDeploymentNotifier(sandbox_deployment_state_tracker,
+                                                                            messaging_service,
+                                                                            _app_health_check_state,
+                                                                            config_json['space_id'],
+                                                                            sandbox_id)
+
         @_app_status_maintainer.on_instance_update_status.connect
         def on_instance_update_status(sender, **kwargs):
             identifier = kwargs.get("identifier")
@@ -415,6 +411,7 @@ def run():
         @_app_health_check_state.on_apps_deployment_complete.connect
         def on_apps_deployment_complete(sender, **kwargs):
             _start_time_updater.on_app_instance_configuration_status_updated()
+            _app_sandbox_end_deployment_notifier.notify_end_deployment()
 
     except Exception as exc:
         logger.exception("Sidecar failed to start")
@@ -424,18 +421,6 @@ def run():
     app.run(host='0.0.0.0', port=4000)
 
 
-def app_request_started(sender, **extra):
-    logger.info("Request Started " + request.endpoint)
-
-
-def app_request_ended(sender, response, **extra):
-    logger.info("Request Ended " + request.endpoint + " with " + response.status)
-
-
-def app_request_exception(sender, exception, **extra):
-    logger.exception("error in request {}".format(str(exception)))
-
-
 def configure_debug():
     global _kub_token_provider
     _kub_token_provider = FakeKubTokenProvider()
@@ -475,12 +460,12 @@ def configure_debug():
     }
 
     config_file_content = {
-        "environment": "evgenyk",
+        "environment": "alberto-b",
         "provider": "kubernetes",
-        "sandbox_id": "l4yequq6g8002",  # can set it to an existing sandbox
+        "sandbox_id": "zrkq78yet8002",  # can set it to an existing sandbox
         "kub_api_address": "https://35.197.69.72",
         "region_name": "eu-west-1",
-        "space_id": "e5a8fb56-f9bd-4d58-b7dc-94c9b1046ea0",
+        "space_id": "d3cddb71-44ba-4f69-9d31-bf828d77ded3",
         "cloud_external_key": "8f70be32-3041-4a00-b96e-29f7a79cc854",
         "apps": apps_json,
         "messaging": messaging_json
@@ -490,5 +475,5 @@ def configure_debug():
 
 
 if __name__ == "__main__":
-    # configure_debug()
+    #configure_debug()
     run()
